//
//  ComposeCurrentAccount.swift
//  Tusker
//
//  Created by Shadowfacts on 8/18/20.
//  Copyright © 2020 Shadowfacts. All rights reserved.
//

import SwiftUI
import Pachyderm

struct ComposeCurrentAccount: View {
    @EnvironmentObject var mastodonController: MastodonController
    @ObservedObject private var preferences = Preferences.shared
    
    var account: Account? {
        mastodonController.account
    }
    
    var body: some View {
        HStack(alignment: .top) {
            ComposeAvatarImageView(url: account?.avatar)
                .frame(width: 50, height: 50)
                .cornerRadius(preferences.avatarStyle.cornerRadiusFraction * 50)
                .accessibility(label: Text(account != nil ? "\(account!.displayName) avatar" : "Avatar"))
            
            if let id = account?.id,
               let account = mastodonController.persistentContainer.account(for: id) {
                VStack(alignment: .leading) {
                    AccountDisplayNameLabel(account: account, textStyle: .title2, emojiSize: 24)
                        .lineLimit(1)
                    
                    Text(verbatim: "@\(account.acct)")
                        .font(.body.weight(.light))
                        .foregroundColor(.secondary)
                        .lineLimit(1)
                }
            }
            
            Spacer()
        }
    }
}

//struct ComposeCurrentAccount_Previews: PreviewProvider {
//    static var previews: some View {
//        ComposeCurrentAccount(account: )
//    }
//}
